-- fix multitablecardinality.sql by denormalizing sales table
-- merging the divname column into sales
-- exadata version - no index on denormalized table

set linesize 1000
set pagesize 1000
set echo on
set termout on
set trimspool on

spool exadenormalize.log

/* DIVISION table with just two rows which point to SALES table.
Idea is that where the DIVNAME column='Mayberry' there are few records in the SALES table.
Where the DIVNAME column='Los Angeles' there are many records.  But the optimizer can't figure this out
given the statistics it has available. */

drop table DIVISION;
drop table SALES;

create table DIVISION (DIVNAME VARCHAR2(2000),DIVNUM NUMBER);
create table SALES (DIVNUM NUMBER,AMOUNT number,padding varchar2(1000)) tablespace tools;

insert into DIVISION values ('Los Angeles',1);
insert into DIVISION values ('Mayberry',2);

commit;

/* SALES table has 1,000,000 1's for LA and one 2 for Mayberry */

declare
  i number;
begin
  i:=1;
  loop
    insert into SALES values (1,1,lpad('X',1000,'X'));
    i:=i+1;
    if (mod(i,10000)=0) then
      commit;
    end if;
    exit when (i > 1000000);
  end loop;
  commit;
  insert into SALES values (2,2,lpad('X',1000,'X'));
  commit;
end;
/

create index SALESINDEX on SALES(DIVNUM)  tablespace tools; 

/* Gather compute stats with max number of histograms all columns and multi column stats on DIVISION table's columns */


select DBMS_STATS.CREATE_EXTENDED_STATS (NULL,'DIVISION','(DIVNAME,DIVNUM)') from dual;
execute DBMS_STATS.GATHER_TABLE_STATS (NULL,'DIVISION',cascade=>TRUE,estimate_percent=>NULL,method_opt=>'FOR ALL COLUMNS SIZE 254');
execute DBMS_STATS.GATHER_TABLE_STATS (NULL,'SALES',cascade=>TRUE,estimate_percent=>NULL,method_opt=>'FOR ALL COLUMNS SIZE 254');

/* Show how long it takes with the wrong plan. */

set timing on

select B.DIVNUM,b.amount 
from DIVISION A,SALES B
where
a.DIVNUM=B.DIVNUM and
A.DIVNAME='Mayberry';

/
/
/
/


set timing off

select * from table(dbms_xplan.display_cursor(null,null,'ALL'));

set timing on

/* Show how long it takes with the index being used correctly.  */

select /*+ INDEX(B SALESINDEX) NO_INDEX_FFS(B SALESINDEX)  */ B.DIVNUM,B.amount 
from DIVISION A,SALES B
where
a.DIVNUM=B.DIVNUM and
A.DIVNAME='Mayberry';

/
/
/
/

set timing off

select * from table(dbms_xplan.display_cursor(null,null,'ALL'));

/* Now build denormalized version of sales and division tables */

drop table SALESWITHDIVNAME;

create table SALESWITHDIVNAME  tablespace tools as
select d.divname,d.divnum,s.amount
from
SALES s,
DIVISION d
where
s.divnum=d.divnum;

/* gather stats with histograms */

execute DBMS_STATS.GATHER_TABLE_STATS (NULL,'SALESWITHDIVNAME',cascade=>TRUE,estimate_percent=>NULL,method_opt=>'FOR ALL COLUMNS SIZE 254');

/* run query with denormalized one table */

set timing on

select DIVNUM,amount
from SALESWITHDIVNAME
where
DIVNAME='Mayberry';

/
/
/
/

set timing off

select * from table(dbms_xplan.display_cursor(null,null,'ALL'));

spool off
